#define WANT_STREAM
#define WANT_MATH

#include <iostream>
#include <vector>
#include <valarray>
#include "newmat.h"	// the basic NEWMAT header files
#include "newmatio.h"    // need matrix output routines
#include "newmatnl.h"    // need matrix newton routines

using namespace std;  //introduces namespace std

class CFA_LL : public LL_D_FI
{
   ColumnVector D;				// derivatives of loglikelihood
   SymmetricMatrix D2;			// second derivatives
   ColumnVector p;				// the parameters
   SymmetricMatrix S;
	
public:
// constructor arguments are covariance matrix and coefficient columnvector
   CFA_LL(const SymmetricMatrix& S_in,
   	const ColumnVector& p_in)
   	: S(S_in), p(p_in) {}  
   	
// more public variables are added here:   	
	Real N2;
//	double N2;
	int nvar, npar, nfac;	// numbers of variables, parameters, and factors
	Matrix Lambda;			// the model is Sigma = Lambda*Lambda' + Delta
	DiagonalMatrix Delta;
	vector< valarray<int> > iL;			// iL and iD are images of Lambda and Delta
	valarray<int> iD;				// that contain the parameter numbers
   	

   void Set(const ColumnVector& p_set) { // set parameter values
      p = p_set;
   }

   bool IsValid();                 // are parameters valid
   Real LogLikelihood();           // return the loglikelihood
   ReturnMatrix Derivatives();     // derivatives of log-likelihood
   ReturnMatrix FI();              // Fisher Information matrix
};

bool CFA_LL::IsValid() {
	 return true;		// could check for positive Delta values here
}

Real CFA_LL::LogLikelihood() {
	Real LL;          // the log likelihood
   
	D.ReSize(npar);			// Note: It is silly and slow to re-size or
	D2.ReSize(npar);		// re-allocated all of these matrices here
   							// they should be global to the class and
	SymmetricMatrix LPL(nvar);	// sized at initialization
	SymmetricMatrix Sigma(nvar);
	SymmetricMatrix SigmaInverse(nvar);
	Matrix SigmaS(nvar,nvar);

	Matrix dFdL(nvar,nfac); 	// derivative of the loglikelihood wrt Lambda
	ColumnVector dFdD(nvar);	// derivative of the loglikelihood wrt Delta
	IdentityMatrix I(nvar);

	Matrix xi(nvar,nfac); 		// Joreskog's intermediates for 
	Matrix alpha(nfac,nfac);  	// second derivatives
	Matrix H(npar,npar);		// which ultimately go into H
	
	int i,j,k,l;
	for (i=1; i <= nvar; i++) {				// this block moves
		for (j=1; j <= nfac; j++) {			// the parameters from p
			if (iL[i][j] != 0) 				// into Lambda and Delta
				Lambda(i,j) = p(iL[i][j]);	// using the index numbers
		}									// in iL and iD
		if (iD[i] != 0)
			Delta(i) = p(iD[i]);
	}
	LPL << Lambda * Lambda.t();		// the usual loglikelihood computations
	Sigma << LPL + Delta;			// for a factor model are here
	SigmaInverse = Sigma.i();
	SigmaS = SigmaInverse * S;

	LL = -N2 * (log(Sigma.Determinant()) + SigmaS.Trace());
// Joreskog's derivatives of LL wrt Lambda and Delta
	dFdL = 2.0 * (SigmaInverse * (Sigma - S) * SigmaInverse) * Lambda;
	dFdD = SP(I, (SigmaInverse * (Sigma - S) * SigmaInverse)).AsColumn();
	
	D = 0;
	for (i=1; i <= nvar; i++) {			// this block moves the derivatives
		for (j=1; j <= nfac; j++) {		// into elements of D
			if (iL[i][j] != 0) 			// as indicated by iL and iD;
				D(iL[i][j]) += dFdL(i,j);	// derivatives wrt the same parameter
		}									// are summed
		if (iD[i] != 0)
			D(iD[i]) += dFdD(i);
	}
	D = -N2*D;

	if (wg) {    // do only if second derivatives wanted
		xi = SigmaInverse * Lambda;		// Joreskog's intermediates for
		alpha = Lambda.t() * xi;		// second derivatives
		H = 0;
		for (i=1; i<=nvar; i++) {			// this block computes & moves the second derivatives
			for (j=1; j<=nvar; j++) {		// into H, summing those for the same
				for (k=1; k<=nfac; k++) {	// parameter number into the same locations of H
					for (l=1; l<=nfac; l++) {
						if ((iL[i][k] > 0) & (iL[j][l] > 0))
							H(iL[i][k],iL[j][l]) += 2*(SigmaInverse(i,j)*alpha(k,l) + xi(i,k)*xi(j,l));
					}
					if ((iL[j][k] > 0) & (iD[i] > 0)) {
						H(iL[j][k],iD[i]) += 2*(SigmaInverse(i,j)*xi(i,k));
						H(iD[i],iL[j][k]) += 2*(SigmaInverse(i,j)*xi(i,k));
					}
				}
				if ((iD[i] > 0) & (iD[j] > 0))
				H(iD[i],iD[j]) += SigmaInverse(i,j)*SigmaInverse(i,j);
			}
		}
		D2 << N2*H;
	}
/*
cout << "loglikelihood " << LL << endl;
   	cout << "parameters " << p;
   	cout << "LL " << LL << endl;
	cout << "D " << D;
	if (wg) cout << "D2 " << D2;
*/	
   return LL;
}

ReturnMatrix CFA_LL::Derivatives() { 
	return D; 
}

ReturnMatrix CFA_LL::FI() {
   if (!wg) cout << endl << "unexpected call of FI" << endl;
   return D2;
}
